/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.conient.datacomponents;

//---IMPORTS---
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JPanel;
import java.awt.GridLayout;
import javax.swing.SwingUtilities;
import uk.org.iscream.cms.util.XMLPacket;

/**
 * Used to display IO style information.
 *
 * This datacomponent is designed to display two attributes
 * side-by-side for data that has an in/out nature. For
 * example, network interface statistics would be suitable.
 *
 * It simply displays the values as Strings in a JTextField
 *
 * @author  $Author: tdb $
 * @version $Id: IODataComponent.java,v 1.1 2003/03/06 12:15:53 tdb Exp $
 */
public class IODataComponent extends VisibleDataComponent {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.1 $";
    
    /**
     * The default length of the JTextField
     */
    protected final int DEFAULT_TEXT_LENGTH = 3;

//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * Creates the component with a friendly name to be 
     * used as label
     *
     * @param name the friendly name
     * @param inAttribute the input data attribute we look after
     * @param outAttribute the output data attribute we look after
     */
    public IODataComponent(String name, String inAttribute, String outAttribute) {
        _name = name;
        _inAttribute = inAttribute;
        // create input item
        _inItem = new JTextField("", DEFAULT_TEXT_LENGTH);
        _inItem.setEditable(false);
        _inItem.setHorizontalAlignment(JTextField.CENTER);
        _inItem.setText("-uninitialised-");
        // create output item
        _outAttribute = outAttribute;
        _outItem = new JTextField("", DEFAULT_TEXT_LENGTH);
        _outItem.setEditable(false);
        _outItem.setHorizontalAlignment(JTextField.CENTER);
        _outItem.setText("-uninitialised-");
        // create label
        _label = new JLabel(_name + ": ");
        _label.setHorizontalAlignment(JLabel.RIGHT);
        // put the in/out items in a panel for layout tidyness
        JPanel io = new JPanel();
        io.setLayout(new GridLayout(2, 2));
        io.add(new JLabel("in", JLabel.CENTER));
        io.add(new JLabel("out", JLabel.CENTER));
        io.add(_inItem);
        io.add(_outItem);
        // add to our panel
        setLayout(new GridLayout(1, 2));
        add(_label);
        add(io);
        setVisible(false);
    }

//---PUBLIC METHODS---

    /**
     * This run method updates any Swing components
     * The setValue() method adds this component
     * to the Swing Event Dispatching Queue to
     * run this method.
     */
    public void run() {
        if(!isVisible()) {
            setVisible(true);
        }
        _inItem.setText(_inCache);
        _outItem.setText(_outCache);
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return _name + "(" + _inAttribute + "/" + _outAttribute + ")";
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

    /**
     * This takes the packet to obtain the value from, it then performs all
     * approriate conversions and adds this class to the Swing Event
     * Dispatching queue.
     *
     * @param packet the XMLPacket to get the data from
     * @throws DataFormatException if there was a problem converting the data for display
     */
    public void setValue(XMLPacket packet) throws DataFormatException {
        String inValue = packet.getParam(_inAttribute);
        String outValue = packet.getParam(_outAttribute);
        try {
            // we'll update both values if either change
            if(!_inCache.equals(inValue) || !_outCache.equals(outValue)) {
                _inCache = inValue;
                _outCache = outValue;
                SwingUtilities.invokeLater(this);
            }
        } catch (Exception e) {
            throw new DataFormatException(inValue + "/" + outValue + " are invalid data types for " + toString());
        }
    }
    
    /**
     * Returns the string showing the packet
     * attribute that the component is looking after
     *
     * @return the packet reference
     */
    public String getPacketAttribute() {
        // we can irritatingly only return one attribute
        // this means we have to do something outside of this
        // class to hold on to the out attribute :/
        return _inAttribute;
    }

//---ATTRIBUTES---

    /**
     * The friendly name of this component
     */
    protected String _name;
    
    /**
     * The input attribute that this component is concerned with
     */
    protected String _inAttribute;
    
    /**
     * The output attribute that this component is concerned with
     */
    protected String _outAttribute;

    /**
     * The friendly label for this component
     */
    protected JLabel _label;
    
    /**
     * Remembers what the last input value was, so we
     * only update if we have to.
     */    
    protected String _inCache = "";
    
    /**
     * Remembers what the last output value was, so we
     * only update if we have to.
     */  
    protected String _outCache = "";
    
    /**
     * The length of the JTextField
     */
    protected int _displayLength;
    
    /**
     * Just a normal label to display our input value as a String
     */
    protected JTextField _inItem;
    
    /**
     * Just a normal label to display our output value as a String
     */
    protected JTextField _outItem;

//---STATIC ATTRIBUTES---

}
