/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.conient.datacomponents;

//---IMPORTS---
import javax.swing.JLabel;
import java.awt.GridLayout;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;
import uk.org.iscream.cms.util.XMLPacket;

/**
 * This is DataComponent specifically for 
 * displaying storage, eg memory, disk and swap.
 * It displays a value/total type display using a JProgessBar.
 *
 * @author  $Author: tdb $
 * @version $Id: StorageDataComponent.java,v 1.7 2003/02/05 19:35:04 tdb Exp $
 */
public class StorageDataComponent extends VisibleDataComponent {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.7 $";
    
//---STATIC METHODS---
    
//---CONSTRUCTORS---

    /**
     * Creates the component with a friendly name to be 
     * used as label
     *
     * @param name the friendly name
     * @param attribute the data attribute we look after
     * @param maxAttribute the data attribute to obtain the maximum value from
     * @param unit the string representation of the units eg, "Mb" or "Kb"
     */
    public StorageDataComponent(String name, String attribute, String maxAttribute, String unit) {
        this(name, attribute, maxAttribute, unit, 1);
    }
    
    /**
     * Creates the component with a friendly name to be 
     * used as label
     *
     * @param name the friendly name
     * @param attribute the data attribute we look after
     * @param maxAttribute the data attribute to obtain the maximum value from
     * @param unit the string representation of the units eg, "Mb" or "Kb"
     * @param divider the amount that the given value should be multiplied by to reach the given units
     */
    public StorageDataComponent(String name, String attribute, String maxAttribute, String unit, int divider) {
        _name = name;
        _attribute = attribute;
        _unit = unit;
        _divider = divider;
        _maxAttribute = maxAttribute;
        _label = new JLabel(name + ": ");
        _label.setHorizontalAlignment(JLabel.RIGHT);
        setLayout(new GridLayout(1, 2));
        _item.setString("-uninitialised-");
        add(_label);
        add(_item);
        setVisible(false);
    }

//---PUBLIC METHODS---
    
    /**
     * This run method updates any Swing components
     * The setValue() method adds this component
     * to the Swing Event Dispatching Queue to
     * run this method.
     */
    public void run() {
        if(!isVisible()) {
            setVisible(true);
        }
        int max = new Double(_maxvalue).intValue();
        _item.setMaximum(max);
        int value = new Double(_value).intValue();
        _item.setValue(max - value);
        _item.setString((max - value) + _unit + "/" + max + _unit + " (" + new Double(_item.getPercentComplete() * 100).intValue() + "%)");
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return _name + "(" + _attribute + ")";
    }
    
//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

   /**
     * This takes the packet to obtain the value from, it then performs all
     * approriate conversions and adds this class to the Swing Event
     * Dispatching queue.
     *
     * @param packet the XMLPacket to get the data from
     * @throws DataFormatException if there was a problem converting the data for display
     */
    public void setValue(XMLPacket packet) throws DataFormatException {
        String value = packet.getParam(_attribute);
        String maxvalue = packet.getParam(_maxAttribute);
        try {
            if((!maxvalue.equals("0")) && (!_cache.equals(value) || !_maxcache.equals(maxvalue))) {
                _cache = value;
                _maxcache = maxvalue;
                _value = Double.parseDouble(_cache) / _divider;
                _maxvalue = Double.parseDouble(_maxcache) / _divider;
                SwingUtilities.invokeLater(this);
            }
        } catch (Exception e) {
            throw new DataFormatException(value + " is an invalid data type for " + toString());
        }
    }        

    /**
     * Returns the string showing the packet
     * attribute that the component is looking after
     *
     * @return the packet reference
     */
    public String getPacketAttribute() {
        return _attribute;
    }

//---ATTRIBUTES---

    /**
     * The friendly name of this component
     */
    protected String _name;
    
    /**
     * The attribute that this component is concerned with
     */
    protected String _attribute;
    
    /**
     * The max attribute that this component is concerned with
     */
    protected String _maxAttribute;
    
    /**
     * The string representation of the units the values
     * are being measured in.
     */
    protected String _unit;
    
    /**
     * The divider to multiply the value to
     * to reach the units.
     */
    protected int _divider;
        
    /**
     * Remebers what the last value was, so we
     * only update if we have to.
     */    
    protected String _cache = "";
    
    /**
     * Remebers what the last value was, so we
     * only update if we have to.
     */    
    protected String _maxcache = "";
    
    /**
     * Holds the integer value we are holding
     */
    protected double _value = 0;
    
    /**
     * Holds the integer value we are holding
     */
    protected double _maxvalue = 0;

    /**
     * The minimum value for the percentage
     */
    protected final int _min = 0;
    
    /**
     * The maximum value for the percentage
     */
    protected int _max = 0;
    
    /**
     * The friendly label for this component
     */
    protected JLabel _label;
    
    /**
     * The progress bar that we will display memory
     * usage data in
     */
    protected JProgressBar _item = new JProgressBar(JProgressBar.HORIZONTAL, _min, _max);
    {
        _item.setStringPainted(true);
    }

//---STATIC ATTRIBUTES---

}
